# -*- coding: utf-8 -*-

from .exceptions import CircularDependencyError

class Node:
    def __init__(self, name, depends, instance):
        self.name = name
        self.depends = set(depends)
        self.instance = instance

def solve(nodes):
    # Returns a list of instances ordered by dependency
    deps = {node.name: node for node in nodes}
    result = []
    while deps:
        # Get a batch of nodes not depending on anything (or originally depending on already resolved nodes)
        batch = {name for name, node in deps.items() if not node.depends}
        if not batch:
            # If there are no such nodes, we have found a circular dependency
            raise CircularDependencyError(deps)
        # Add instances tied to the resolved keys to the result and remove resolved keys from the dependecy map
        for name in batch:
            result.append(deps[name].instance)
            del deps[name]
        # Remove resolved keys from the dependencies of yet unresolved nodes
        for node in deps.values():
            node.depends -= batch
    return result
